#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/time.h>

#include "camconfig.h"
#include "video.h"
#include "filter.h"

typedef struct rand_filter_st {
  int colored_pixels;
  int num_perline;
} RandFilter;

static
void log_msg( const char *pattern, ... ) {
  char decent_buffer[ 1024 ];
  va_list ap;
  time_t curtime;

  va_start( ap, pattern );
  vsnprintf( decent_buffer, sizeof( decent_buffer ), pattern, ap );
  va_end( ap );

  time( &curtime );
  fprintf( stderr, "%s: (randfilter): %s\n", ctime( &curtime ),decent_buffer );
}

/*
 * filter_init:  Standard filter initialization routine.
 *
 * Filter variables:  randmod_perline = max # of random pixels per line
 *                    randmod_colordpix = Use colored pixels
 */

void *filter_init( CamConfig *ccfg, int filter_num ){
  RandFilter *res;

  if( (res = malloc( sizeof( *res ))) == NULL ){
    log_msg( "FATAL!  Could not allocate space for random filter!\n" );
    return NULL;
  }

  res->num_perline = camconfig_query_def_int( ccfg, "randmod_perline", 20 );
  res->colored_pixels = camconfig_query_def_int( ccfg, "randmod_coloredpix",1);
  return res;
}

/*
 * filter_deinit:  Standard filter deinit routine 
 */

void filter_deinit( void *filter_dat ){
  RandFilter *rfilt = filter_dat;

  free( rfilt );
}

void filter_func( char *in_data, char **out_data, void *cldat, 
		  const Video_Info *vinfo_in, Video_Info *vinfo_out )
{
  RandFilter *rfilt = cldat;
  int rowspan, i, j, randval;
  unsigned char *cp, *outp;

  *vinfo_out = *vinfo_in;
  *out_data = in_data;

  if( vinfo_in->is_black_white ) { /* UNTESTED */
    rowspan = vinfo_in->width;
    
    for( i=0, cp = in_data; i< vinfo_in->height; i++, cp += rowspan ) {
      randval = random() % 100;
      for( j=0; j< randval; j++ ){
	outp = cp + (random() % vinfo_in->width );
	*outp = random() % 256;
      }
    }
  } else {
    /* Pick random spots per line */
    rowspan = vinfo_in->width * 3;

    for( i=0, cp = in_data; i< vinfo_in->height; i++, cp += rowspan ){
      randval = random() % rfilt->num_perline;
      if( rfilt->colored_pixels ) {
	for( j=0; j< randval; j++ ){
	  outp = cp + 3*(random() % vinfo_in->width );
	  *(outp + 0) = random() % 255;
	  *(outp + 1) = random() % 255;
	  *(outp + 2) = random() % 255;
	}
      } else {
	for( j=0; j< randval; j++ ){
	  outp = cp + 3*(random() % vinfo_in->width );
	  *(outp + 0) = 255 - *(outp + 0 );
	  *(outp + 1) = 255 - *(outp + 1 );
	  *(outp + 2) = 255 - *(outp + 2 );
	}
      }
    }
  }
}

void filter_validation(){
  Filter_Init_Func init = filter_init;
  Filter_Deinit_Func deinit = filter_deinit;
  Filter_Func_Func func = filter_func;

  if( init != NULL && deinit != NULL && func != NULL ) return;
}
